@@ -63,9 +63,9 @@ SEXP stri_startswith_fixed(SEXP str, SEXP pattern, SEXP from, SEXP negate, SEXP
 {
     bool negate_1 = stri__prepare_arg_logical_1_notNA(negate, "negate");
     uint32_t pattern_flags = StriContainerByteSearch::getByteSearchFlags(opts_fixed);
-    PROTECT(str = stri_prepare_arg_string(str, "str"));
-    PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
-    PROTECT(from = stri_prepare_arg_integer(from, "from"));
+    PROTECT(str = stri__prepare_arg_string(str, "str"));
+    PROTECT(pattern = stri__prepare_arg_string(pattern, "pattern"));
+    PROTECT(from = stri__prepare_arg_integer(from, "from"));
 
     STRI__ERROR_HANDLER_BEGIN(3)
     int vectorize_length = stri__recycling_rule(true, 3,
@@ -140,9 +140,9 @@ SEXP stri_endswith_fixed(SEXP str, SEXP pattern, SEXP to, SEXP negate, SEXP opts
 {
     bool negate_1 = stri__prepare_arg_logical_1_notNA(negate, "negate");
     uint32_t pattern_flags = StriContainerByteSearch::getByteSearchFlags(opts_fixed);
-    PROTECT(str = stri_prepare_arg_string(str, "str"));
-    PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
-    PROTECT(to = stri_prepare_arg_integer(to, "to"));
+    PROTECT(str = stri__prepare_arg_string(str, "str"));
+    PROTECT(pattern = stri__prepare_arg_string(pattern, "pattern"));
+    PROTECT(to = stri__prepare_arg_integer(to, "to"));
 
     STRI__ERROR_HANDLER_BEGIN(3)
     int vectorize_length = stri__recycling_rule(true, 3,
